﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProgressForm
{
    public class Progress
    {
        private bool doEnd = false;
        private bool doCancel = false;
        private int currentNum = 0;
        private int maxNum = 100;

        private static bool isCanceled = false;
        private static System.Windows.Forms.Timer watcher = new System.Windows.Forms.Timer() { Enabled = false, Interval = 100 };
        private static TaskDialogProgressBar progressBar = new TaskDialogProgressBar() { State = TaskDialogProgressBarState.Normal };
        private static TaskDialogButton cancelButton = new TaskDialogButton() { Text = "キャンセル" };
        private static TaskDialogButton okButton = new TaskDialogButton() { Text = "OK", Visible = false };
        private TaskDialogPage dialogPage = new TaskDialogPage() {
            Heading = "処理中です。",
            Icon = TaskDialogIcon.Warning,
            ProgressBar = progressBar,
            Buttons =
            {
                okButton,
                cancelButton
            }
        };

        static Progress? progress = null;

        private static void initProgress(bool isForce = false)
        {
            if (progress == null)
            {
                progress = new Progress();
                isCanceled = false;
                SetMax(100);
                SetProgress(0);
                progress.doEnd = false;
                progress.doCancel = false;
            }
            else if(isForce) {
                isCanceled = false;
                SetMax(100);
                SetProgress(0);
                progress.doEnd = false;
                progress.doCancel = false;
            }
        }

        public static void Start( Action<object?> function, Form? owner = null, object? methodArg = null )
        {
            initProgress(true);

            if(progress != null) {
                watcher.Enabled = true;
                watcher.Tick += (sender, eventArg) =>
                {
                    try
                    {
                        if (progress.doCancel)
                        {
                            watcher.Enabled = false;
                            cancelButton.PerformClick();
                        }
                        if (progress.doEnd)
                        {
                            watcher.Enabled = false;
                            okButton.PerformClick();
                        }
                        if(progress.dialogPage.ProgressBar != null)
                        {
                            progress.dialogPage.ProgressBar.Value = progress.currentNum;
                            progress.dialogPage.ProgressBar.Maximum = progress.maxNum;
                        }
                        progress.dialogPage.Heading = string.Format("処理中です。 {0:#,0} / {1:#,0} 件", progress.currentNum, progress.maxNum);
                    }
                    catch (Exception)
                    { }
                };

                Task.Run(() => function(methodArg));

                TaskDialogButton retButton;
                if (owner != null)
                {
                    retButton = TaskDialog.ShowDialog(owner, progress.dialogPage);
                }
                else
                {
                    retButton = TaskDialog.ShowDialog(progress.dialogPage);
                }
                if (retButton == cancelButton)
                {
                    isCanceled = true;
                }
            }
        }

        public static void SetMax(int maxNum)
        {
            initProgress();
            if (progress != null)
            {
                progress.maxNum = maxNum;
            }

        }

        public static void SetProgress(int currentNum)
        {
            initProgress();
            if (progress != null)
            {
                progress.currentNum = currentNum;
            }
        }

        public static bool IsCancelled()
        {
            return isCanceled;
        }

        public static void Cancel()
        {
            if (progress != null)
            {
                progress.doCancel = true;
            }
        }

        public static void End()
        {
            if (progress != null)
            {
                progress.doEnd = true;
            }
        }
    }
}
